unit Smartqry;
{Created by:  Brian D. Butcher
 Compuserve:  73737,164

You may use this component in your applications at no charge provided that you
notify me via Compuserve mail.

Description:  SmartQuery is a new component derived from TQuery that lets you
              build a SQL Select query by setting properties.

Installation: You can install SmartQuery by selecting "Install Components..."
              from the Options menu in Delphi.  Then click the "Add" button and
              select this file.  The SmartQuery component will appear on the
              "Data Access" toolbar.

Using SmartQuery:
       1. Create a SmartQuery component on your form by double-clicking the
          SmartQuery Icon.
       2. View the Object Inspector window
       3. Set the DatabaseName property to your database.  See the Help screen
          for TQuery if you need instructions.
       4. Set the Recordsource Property.  Usually this is the name of the table
          that you will be searching.  Example: tblCustomer.  It can also be a
          more complex recordsource such as an INNER JOIN, etc.
       5. Optional:  Set the FieldList property to select the fields you want to
          retrieve.  Example: Last,First.  Leave blank to include all fields in
          the record source.
       6. Optional:  Set the Criteria property to limit the search to matching
          records.  Example:  ZipCode>=90000.  You can use create more complex
          criteria by using SQL expressions and operators.
       7. Optional:  Set the Sortby property to set the sort order.  Example:
          Last,First.

       After completing step 4, you can view the SQL statement by viewing the
       SQL property of the SmartQuery.

Other Notes:  I have found it useful to derive new components from SmartQuery to
              create "shortcut" components to my databases.  For example, I
              created a component called CustQry that has the DatabaseName and
              Recordsource properties filled in automatically (in the Create
              method).  To use my customer file, I just drop a CustQry component
              on my form and set the criteria and/or sort order.}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables;

type
  TSmartQuery = class(TQuery)
  private
    FFieldList:string;
    FRecordSource:String;
    FCriteria:string;
    FSortBy:string;
    Procedure UpdateSQL;
  protected
    Procedure SetRecordSource(newvalue:string);
    Procedure SetCriteria(newvalue:string);
    Procedure SetSortBy(newvalue:string);
    Procedure SetFieldList(newvalue:string);
  public
  published
    property RecordSource:string read FRecordSource write SetRecordSource;
    Property Criteria:string read FCriteria write SetCriteria;
    property SortBy:string read FSortby write SetSortBy;
    property FieldList:string read FFieldList write SetFieldList;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Data Access', [TSmartQuery]);
end;

procedure TSmartQuery.UpdateSQL;
var
  sqlstr:string;
  oldactivestate:boolean;
begin
  oldactivestate:=active;
  if FRecordSource<>'' then
    begin
      sqlstr:='SELECT ';
      if FFieldList<>'' then
        sqlstr:=sqlstr + FFieldList
      else
        sqlstr:=sqlstr + '*';
      sqlstr:=sqlstr + ' FROM ' + FRecordSource;
      if FCriteria<>'' then
        sqlstr:=sqlstr + ' WHERE ' + FCriteria;
      if FSortBy<>'' then
        sqlstr:=sqlstr + ' ORDER BY ' + FSortBy;
      close;
      sql.clear;
      sql.add(sqlstr);
    end
  else
    sql.clear;
  active:=oldactivestate;
end;

procedure TSmartQuery.SetRecordSource(newvalue:string);
begin
  FRecordSource:=newvalue;
  UpdateSQL;
end;

Procedure TSmartQuery.SetCriteria(newvalue:string);
begin
  FCriteria:=newvalue;
  UpdateSQL;
end;

Procedure TSmartQuery.SetSortBy(newvalue:string);
begin
  FSortBy:=newvalue;
  UpdateSQL;
end;

Procedure TSmartQuery.SetFieldList(newvalue:string);
begin
  FFieldList:=newvalue;
  UpdateSQL;
end;

end.
